=begin
(c) TIG 2010
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided the above
copyright notice appear in all copies.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
___________________________________________________________________________
Script:
  ##hideEverythingElse.rb
place it in the ../Plugins/ folder...
___________________________________________________________________________
Usage:
This script adds a new function to the 'View' menu.
  'Editing: Hide Context'
It 'toggles'.
If it is active it is shown 'ticked'.
If it is inactive it is shown 'unticked'.
The default setting is 'inactive'.
The last setting is remembered with the saved model across sessions.
When it is 'active' if you edit a component-instance or a group then 
everything else in its context is hidden from view: unlike the built-in 
'View'>'Component Edit'>'Hide Rest of Model' and 
'View'>'Component Edit'>'Hide Similar Instances', 
which will either show OR hide EVERYTHING both inside AND outside of the 
editing context.
'Editing: Hide Context' is limited to hiding only objects in the edited 
instance's context.
This may be more useful than 'Hide Rest of Model' if there is something 
outside of the context that you still want to see [e.g. to snap to], but 
you do want objects in the immediate context to be hidden temporarily as 
they obscure your view...
Remember to set 'View'>'Hidden Geometry' to 'off' for an unobstructed view.
Note that because it is a menu-item you can shortcut-key to it.
___________________________________________________________________________
Donations:
by PayPal to info @ revitrev.org
___________________________________________________________________________
Version:
1.0 20101019 First Release.
1.1 20101020 Recast as 'Editing: Hide Context',hides editing context ONLY.
___________________________________________________________________________
=end
#####################
require 'sketchup.rb'
#####################
#####
class HideEverythingElseInstanceObserver < Sketchup::InstanceObserver
  def onOpen(instance)
    model=Sketchup.active_model
    if model.get_attribute("HideEverythingElse","toggle","")==""
      model.set_attribute("HideEverythingElse","toggle","off")
    end#if
    if model.get_attribute("HideEverythingElse","toggle","")=="on"
      $hideEverythingElseArray.each{|e|e.hidden=false if e.valid?} if $hideEverythingElseArray
      $hideEverythingElseArray=[]
      entsa=instance.parent.entities.to_a - [instance]
      entsa.each{|e|
        if e.valid? and not e.hidden?
          $hideEverythingElseArray << e
          e.hidden=true
        end#if
      }
    end#if
    model.active_view.invalidate
  end#def
  def onClose(instance)
    model=Sketchup.active_model
    $hideEverythingElseArray.each{|e|e.hidden=false if e.valid?} if $hideEverythingElseArray
    $hideEverythingElseArray=[]
    model.active_view.invalidate
  end#def
end#class
###########################################################################
$hideEverythingElseInstanceObserver=HideEverythingElseInstanceObserver.new
###########################################################################

### update as defs / instances added
class HideEverythingElseDefinitionsObserver < Sketchup::DefinitionsObserver
  def onComponentAdded(definitions, definition)
    definitions.each{|d|
      d.instances.each{|i|
        begin
          i.remove_observer($hideEverythingElseInstanceObserver)
        rescue
          ###
        end
        i.add_observer($hideEverythingElseInstanceObserver)
      }
    }
  end#def
  def onComponentTypeChanged(definitions, definition)
    definitions.each{|d|
      d.instances.each{|i|
        begin
          i.remove_observer($hideEverythingElseInstanceObserver)
        rescue
          ###
        end
        i.add_observer($hideEverythingElseInstanceObserver)
      }
    }
  end#def
end#class
###########################################################################
$hideEverythingElseDefinitionsObserver=HideEverythingElseDefinitionsObserver.new
###########################################################################

### model observer
class HideEverythingElseModelObserver < Sketchup::ModelObserver
  #def onActivePathChanged(model)
    #$hideEverythingElseArray.each{|e|e.hidden=false if e.valid?} if $hideEverythingElseArray
  #end
  def onPreSaveModel(model)
    $hideEverythingElseArray.each{|e|e.hidden=false if e.valid?} if $hideEverythingElseArray
  end
  def onSaveModel(model)
    $hideEverythingElseArray.each{|e|e.hidden=true if e.valid?} if $hideEverythingElseArray
  end
end
###########################################################################
$hideEverythingElseModelObserver=HideEverythingElseModelObserver.new
###########################################################################

### observe for added instances because DefinitionObserver is FUBAR on making 'copies' !!!
class HideEverythingElseEntitiesObserver < Sketchup::EntitiesObserver
  def onElementAdded(entities, entity)
    if entity.class==Sketchup::Group or entity.class==Sketchup::ComponentInstance or entity.class==Sketchup::Image
      begin
        entity.remove_observer($hideEverythingElseInstanceObserver)
      rescue
        ###
      end
      entity.add_observer($hideEverythingElseInstanceObserver)
    end#if
  end
end#class
###########################################################################
$hideEverythingElseEntitiesObserver=HideEverythingElseEntitiesObserver.new
###########################################################################


### we must load observers at start #######################################
#######--------------------------------------------------------------------
def addHideEverythingElseObserver()
  model=Sketchup.active_model
  if model.get_attribute("HideEverythingElse","toggle","")==""
    model.set_attribute("HideEverythingElse","toggle","off")
  end#if
  if model.get_attribute("HideEverythingElse","toggle","")=="on"
    model.definitions.add_observer($hideEverythingElseDefinitionsObserver)
    model.definitions.each{|d|
      d.entities.add_observer($hideEverythingElseEntitiesObserver)
      d.instances.each{|i|i.add_observer($hideEverythingElseInstanceObserver)}
    }
    model.entities.add_observer($hideEverythingElseEntitiesObserver)
    model.add_observer($hideEverythingElseModelObserver)
  end#
end#def
####### run at load
addHideEverythingElseObserver()
#######--------------------------------------------------------------------

#### also ensure it works on new/menu models etc with an AppObserver
class HideEverythingElseAppObserver < Sketchup::AppObserver
  def onNewModel(model)  ### from menu or app
    addHideEverythingElseObserver()
  end
  def onOpenModel(model) ### from menu
    addHideEverythingElseObserver()
  end
  def onQuit()
    $hideEverythingElseArray.each{|e|e.hidden=false if e.valid?} if $hideEverythingElseArray
  end
end
# Attach the observer at kick-off
Sketchup.add_observer(HideEverythingElseAppObserver.new)
########################################################

###########################################################################
#### tool ##############
def hideEverythingElse()
  model=Sketchup.active_model
  defs=model.definitions
  if model.get_attribute("HideEverythingElse","toggle","")==""
    model.set_attribute("HideEverythingElse","toggle","off")
  end#if
  if model.get_attribute("HideEverythingElse","toggle","")=="off"
    begin
      defs.remove_observer($hideEverythingElseDefinitionsObserver)
    rescue
      ###
    end
    model.definitions.add_observer($hideEverythingElseDefinitionsObserver)
    ###
    defs.each{|d|
      begin
        d.entities.remove_observer($hideEverythingElseEntitiesObserver)
      rescue
        ###
      end
      d.entities.add_observer($hideEverythingElseEntitiesObserver)
      ###
      d.instances.each{|i|
        begin
          i.remove_observer($hideEverythingElseInstanceObserver)
        rescue
          ###
        end
        i.add_observer($hideEverythingElseInstanceObserver)
      }
    }
    begin
      model.entities.remove_observer($hideEverythingElseEntitiesObserver)
    rescue
      ###
    end
    model.entities.add_observer($hideEverythingElseEntitiesObserver)
    ###
    model.set_attribute("HideEverythingElse","toggle","on")
  else ### undo anything current
    begin
      defs.remove_observer($hideEverythingElseDefinitionsObserver)
    rescue
      ###
    end
    $hideEverythingElseArray.each{|e|e.hidden=false if e.valid?} if $hideEverythingElseArray
    defs.each{|d|
      begin
        d.entities.remove_observer($hideEverythingElseEntitiesObserver)
      rescue
        ###
      end
      ###
      begin
        d.instances.each{|i|i.remove_observer($hideEverythingElseInstanceObserver)}
      rescue
        ###
      end
    }
    begin
      model.entities.remove_observer($hideEverythingElseEntitiesObserver)
    rescue
      ###
    end
    ###
    model.set_attribute("HideEverythingElse","toggle","off")
  end#if
end#def
############################################################

###########################################################################
### menu
if not file_loaded?(File.basename(__FILE__))
  menu=UI.menu("View").add_item("Editing: Hide Context"){hideEverythingElse()}
  UI.menu.set_validation_proc(menu){
    if Sketchup.active_model.get_attribute("HideEverythingElse","toggle","")=="off"
      MF_UNCHECKED
    else
      MF_CHECKED
    end#if
  }
end
file_loaded(File.basename(__FILE__))
###########################################################################